﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using SuperMap.Desktop;
using SuperMap.Desktop.UI;
using Application = SuperMap.Desktop.Application;

namespace SuperMap.Desktop.Samples
{
    /// <summary>
    /// Inherit from FormBaseChild. The default is the desktop child window.
    /// </summary>
    public partial class FormSearch : FormBaseChild
    {
        private Boolean m_isTextBoxTipShowing;

        public IContextMenu contextMenu;
        public DataGridView DataGrid
        {
            get { return m_dataGridViewResult; }
        }

        //Current selected function
        private IBaseItem m_currentBaseItem;
        public IBaseItem CurrentBaseItem
        {
            get { return m_currentBaseItem; }
        }

        public FormSearch()
        {
            InitializeComponent();
            m_isTextBoxTipShowing = true;
            m_textBoxFilter_Validated(null,null);
            //Initialize the context menu
            contextMenu = Application.ActiveApplication.MainForm.ContextMenuManager["contextMenu"];
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
        }

        void m_textBoxFilter_GotFocus(object sender, EventArgs e)
        {
            if (m_isTextBoxTipShowing)
            {
                m_textBoxFilter.Text = String.Empty;
                m_textBoxFilter.ForeColor = Color.Black;
            }
        }

        void m_textBoxFilter_Validated(object sender, EventArgs e)
        {
            if (String.IsNullOrEmpty(m_textBoxFilter.Text))
            {
                m_isTextBoxTipShowing = true;
                m_textBoxFilter.ForeColor = Color.LightGray;
                m_textBoxFilter.Text = "Please enter the key words, such as \"Raster\"";
            }
            else
            {
                m_isTextBoxTipShowing = false;
            }
        }

        private void m_buttonSearch_Click(object sender, EventArgs e)
        {
            try
            {
                m_dataGridViewResult.Rows.Clear();
                String filter=m_textBoxFilter.Text.Trim();
                if (filter != "")
                {
                    //Search the functions on Ribbon
                    Int16 count = 0;
                    IRibbonManager ribbonManager = Application.ActiveApplication.MainForm.RibbonManager;

                    var items = ribbonManager.SearchItem(filter);
                    foreach (var baseItem in items)
                    {
                        if (baseItem != null)
                        {
                        String baseItemType = String.Empty;
                        String description = String.Empty;
                        Boolean isEnable = false;
                            if (baseItem is IButtonGallery)
                        {
                            baseItemType = "IButtonGallery";
                            if ((baseItem as IButtonGallery).Tip != null)
                            {
                                description = (baseItem as IButtonGallery).Tip.Text;
                            }
                            isEnable = baseItem.CtrlAction.Enable();
                        }
                            else if (baseItem is IButtonDropDown)
                        {
                            baseItemType = "IButtonDropDown";
                            if ((baseItem as IButtonDropDown).Tip != null)
                            {
                                description = (baseItem as IButtonDropDown).Tip.Text;
                            }
                            isEnable = baseItem.CtrlAction.Enable();
                        }
                            else if (baseItem is IButton)
                        {
                            baseItemType = "IButton";
                            if ((baseItem as IButton).Tip != null)
                            {
                                description = (baseItem as IButton).Tip.Text;
                            }
                            isEnable = baseItem.CtrlAction.Enable();
                        }
                            else if (baseItem is ICheckBox)
                        {
                            baseItemType = "ICheckBox";
                            if((baseItem as ICheckBox).Tip != null)
                            {
                                description = (baseItem as ICheckBox).Tip.Text;
                            }
                            isEnable = baseItem.CtrlAction.Enable();
                        }

                        //Add the results to the list
                        if (!String.IsNullOrEmpty(baseItemType))
                        {
                            count++;
                            String strEnable = isEnable ? "Available" : "Unavailable";

                                IBaseItem tabItem = baseItem;
                            while (tabItem.RibbonContainer!=null)
                            {
                                tabItem = tabItem.RibbonContainer;
                            }
                            String tabString = (tabItem as IRibbonTab).Text;
                            PluginInfo pluginInfo = (baseItem.CtrlAction as CtrlAction).PluginInfo;
                            String pluginName=null;
                            if (pluginInfo!=null)
                            {
                                pluginName = (baseItem.CtrlAction as CtrlAction).PluginInfo.Name;
                            }
                            Int32 index = m_dataGridViewResult.Rows.Add(new Object[] { count, baseItem.Text, description ,baseItemType, strEnable, tabString, pluginName });
                            m_dataGridViewResult.Rows[index].Tag = baseItem;
                        }
                        }

                    }
                }
            }
            catch(Exception ex)
            {
                //If there is an exception, it will be displayed on the output window
                Application.ActiveApplication.Output.Output(ex.Message, InfoType.Exception);
                Application.ActiveApplication.Output.Output(ex.StackTrace, InfoType.Exception);
            }
        }

        private void m_textBoxFilter_KeyDown(object sender, KeyEventArgs e)
        {
            try
            {
                //Input the key words and search automatically
                if (e.KeyCode == Keys.Enter)
                {
                    m_buttonSearch_Click(this.m_buttonSearch, new EventArgs());
                }
            }
            catch (System.Exception ex)
            {
                SuperMap.Desktop.Application.ActiveApplication.Output.Output(ex.StackTrace, InfoType.Exception);
            }
        }

        /// <summary>
        /// Right click the list to pop up context menu. Run the current function.
        /// </summary>
        private void m_dataGridViewResult_CellMouseClick(object sender, DataGridViewCellMouseEventArgs e)
        {
            try
            {
                if (e.RowIndex >= 0)
                {
                    m_dataGridViewResult.ClearSelection();
                    m_dataGridViewResult.Rows[e.RowIndex].Selected = true;
                    m_currentBaseItem = m_dataGridViewResult.Rows[e.RowIndex].Tag as IBaseItem;

                    if (e.Button == MouseButtons.Right)
                    {
                        contextMenu.Show(Cursor.Position.X, Cursor.Position.Y);
                    }
                }  
            }
            catch (System.Exception ex)
            {
                SuperMap.Desktop.Application.ActiveApplication.Output.Output(ex.StackTrace, InfoType.Exception);
            }       
        }
    }
}
